/*
 *  Copyright (c) 2008,2010 Cyrille Berger <cberger@cberger.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * either version 2, or (at your option) any later version of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

// C++ Headers
#include <iostream>
#include <fstream>
#include <cstdlib>

#include <GTLCore/CompilationMessages.h>
#include <GTLCore/Type.h>

// OpenRijn Headers
#include <OpenRijn/Sketch.h>
#include <OpenRijn/Version.h>
#include <GTLFragment/LibrariesManager.h>
#include <OpenRijn/LibrariesManager.h>

void printVersion()
{
  std::cout << OpenRijn::LibraryShortName() << " - " << OpenRijn::LibraryName() << " - " << OpenRijn::LibraryVersionString() << std::endl;
  std::cout << OpenRijn::LibraryCopyright() << std::endl;
  std::cout << "Rijn Version : " << OpenRijn::LanguageVersion() << std::endl;
}
void printHelp()
{
  std::cout << "Usage : shivac [option] fileName.shiva" << std::endl;
  std::cout << std::endl;
  std::cout << "Options : " << std::endl;
  std::cout << "  -S --asm-source         print the assembly source code generated" << std::endl;
  std::cout << "  -o --output [filename]  save the output to a file" << std::endl;
  //   std::cout << "  -L --module-dir   add a location where to find modules" << std::endl;
  std::cout << std::endl;
  std::cout << "  -h --help               print this message" << std::endl;
  std::cout << "  -v --version            print the version information" << std::endl;
}
#define ARG_IS(a,b) argv[ai] == GTLCore::String(a) or argv[ai] == GTLCore::String(b)

int main(int argc, char** argv)
{
  GTLCore::String fileName = "";
  GTLCore::String output = "";
  bool showAssembly = false;
  for(int ai = 1; ai < argc; ai++)
  {
    if(ARG_IS("-h","--help"))
    {
      printHelp();
      return EXIT_SUCCESS;
    } else if(ARG_IS("-v","--version"))
    {
      printVersion();
      return EXIT_SUCCESS;
    } else if(ARG_IS("-S","--asm-source")) {
      showAssembly = true;
    } else if(ARG_IS("-o","--output")) {
      if( ai == argc )
      {
        std::cerr << "Expected file name after -o or --output." << std::endl;
        return EXIT_FAILURE;
      } else {
        ++ai;
        output = argv[ai];
      }
    } else if(ARG_IS("-L", "--module-dir")) {
      if( ai == argc )
      {
        std::cerr << "Expected directory after -L --module-dir." << std::endl;
        return EXIT_FAILURE;
      } else {
        ++ai;
        GTLFragment::LibrariesManager::instance()->addDirectory(argv[ai]);
        OpenRijn::LibrariesManager::instance()->addDirectory(argv[ai]);
      }
    } else {
      if( ai != argc - 1)
      {
        std::cerr << "Invalid command line parameters." << std::endl;
      } else {
        fileName = argv[ai];
      }
    }
  }
  if( fileName == "")
  {
    printHelp();
  } else {
    GTLCore::String source;
    std::ifstream in;
    in.open(fileName.c_str() );
    if(not in)
    {
      std::cerr << "Impossible to open file " << fileName << std::endl;
      return EXIT_FAILURE;
    }
    std::string str;
    std::getline(in,str);
    while ( in ) {
      source += str;
      source += "\n";
      std::getline(in,str);
    }
    OpenRijn::Sketch p;
    p.setSource( source );
    p.compile();
    if(not p.isCompiled())
    {
      std::cout << "Error: " << std::endl << p.compilationMessages().toString() << std::endl;
      return EXIT_FAILURE;
    }
    if( showAssembly )
    {
      std::cout << p.asmSourceCode();
    }
  }
  return EXIT_SUCCESS;
}
